package org.runetracker;

import java.io.IOException;
import java.io.Serializable;
import java.io.StreamCorruptedException;

/**
 * Holds settings for the application's runtime.
 * 
 * @author Cynthia of irc.rscript.org
 */
public class Settings implements Serializable {
	private static final long serialVersionUID = 1L;

	private boolean autoUpdate = true;

	private boolean autoUpdateTwiceDaily = false;

	/**
	 * Seconds after midnight, user's local time, at which the RuneTracker
	 * Updater should automatically update the user's data.
	 */
	private int autoUpdateTime;

	private String displayName = "";

	/**
	 * Constructs a new <tt>Settings</tt> object with an automatic update time
	 * of <code>0</code> (midnight, user's local time) and an empty RuneScape
	 * display name.
	 */
	public Settings() {}

	/**
	 * Returns whether the RuneTracker Updater should automatically update the
	 * user's data.
	 * 
	 * @return whether the RuneTracker Updater should automatically update the
	 *         user's data
	 */
	public boolean isAutoUpdate() {
		return autoUpdate;
	}

	/**
	 * Sets whether the RuneTracker Updater should automatically update the
	 * user's data.
	 * 
	 * @param autoUpdate
	 *            Whether the RuneTracker Updater should automatically update
	 *            the user's data.
	 */
	public void setAutoUpdate(boolean autoUpdate) {
		this.autoUpdate = autoUpdate;
	}

	/**
	 * Returns whether the RuneTracker Updater should automatically update the
	 * user's data twice a day.
	 * 
	 * @return whether the RuneTracker Updater should automatically update the
	 *         user's data twice a day
	 */
	public boolean isAutoUpdateTwiceDaily() {
		return autoUpdateTwiceDaily;
	}

	/**
	 * Sets whether the RuneTracker Updater should automatically update the
	 * user's data twice a day.
	 * 
	 * @param autoUpdateTwiceDaily
	 *            Whether the RuneTracker Updater should automatically update
	 *            the user's data twice a day.
	 */
	public void setAutoUpdateTwiceDaily(boolean autoUpdateTwiceDaily) {
		this.autoUpdateTwiceDaily = autoUpdateTwiceDaily;
	}

	/**
	 * Returns the number of seconds after midnight, user's local time, at which
	 * the RuneTracker Updater should automatically update the user's data.
	 * 
	 * @return the number of seconds after midnight, user's local time, at which
	 *         the RuneTracker Updater should automatically update the user's
	 *         data
	 */
	public int getAutoUpdateTime() {
		return autoUpdateTime;
	}

	/**
	 * Sets the number of seconds after midnight, user's local time, at which
	 * the RuneTracker Updater should automatically update the user's data.
	 * 
	 * @param autoUpdateTime
	 *            The number of seconds after midnight, user's local time, at
	 *            which the RuneTracker Updater should automatically update the
	 *            user's data. This value may not be outside of the range [0,
	 *            86400[.
	 */
	public void setAutoUpdateTime(final int autoUpdateTime) {
		if (autoUpdateTime < 0 || autoUpdateTime >= 86400)
			throw new IllegalArgumentException("automatic update time must be between 0 inclusive and 86400 exclusive");
		this.autoUpdateTime = autoUpdateTime;
	}

	/**
	 * Returns the user's RuneScape display name. The returned <tt>String</tt>
	 * is never <code>null</code>, but may be empty.
	 * 
	 * @return the user's RuneScape display name
	 */
	public String getDisplayName() {
		return displayName;
	}

	/**
	 * Sets the user's RuneScape display name. Setting a name different from
	 * that returned by <code>getDisplayName()</code> does not request a name
	 * change on RuneTracker.
	 * 
	 * @param displayName
	 *            The user's RuneScape display name. This parameter may not be
	 *            <code>null</code>, but may be empty.
	 */
	public void setDisplayName(String displayName) {
		if (displayName == null)
			throw new NullPointerException();
		this.displayName = displayName;
	}

	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException {
		in.defaultReadObject();
		if (autoUpdateTime < 0 || autoUpdateTime >= 86400)
			throw new IllegalArgumentException("automatic update time must be between 0 inclusive and 86400 exclusive");
		if (displayName == null)
			throw new StreamCorruptedException("display name may not be null");
	}
}
